// AUTHOR:  Sarah D. Groulx	LAST MODIFIED: June 29th, 1998
//
//========================================================================
//  An example of a User class is given below:                          //
//========================================================================
#include <iostream.h>
#include <stdio.h>
#include <TFile.h>
#include <TCanvas.h>
#include <TMCParticle.h>
#include <TMath.h>
#include <TPythia.h>
#include <TSystem.h>
#include <TROOT.h>
#include <ATLFast.h>
#include <ATLFMCMaker.h>
#include <ATLFMiscMaker.h>
#include <ATLFClusterMaker.h>
#include <TRint.h>
#include <TH1.h>
#include <stdlib.h>
#include "User.h"
//////////////////////////////////////////////////////////////////////////
//                                                                      //
// User                                                                 //
//                                                                      //
// Example of a user class to perform user specific tasks when running  //
// the ATLfast program.                                                 //
//                                                                      //
// This class illustrates some simple uses of the TTree class.  Here,   //
// the TTree tree has one branche: a W branch which contains the        //
// kinematics of the W originating from Drell-Yan W prodection; and     //
// the kinematics of the first two children of the W.                   //
//                                                                      //
// The variables stored in this tree can be plotted using PtPlot_W.C    // 
// which currently plots the child particle codes, the Pt of all        // 
// children, the Pt of muon children, and the particle code of the      //
// found W.                                                             //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

//* Implementation of the User Class
ClassImp(User)

//* These are variables used to fill the W Branch (W_Branch) and the
//* Child TMCParticle members of the of the TTree tree, respectively. 

TMCParticle *particle;
TMCParticle *child1;
TMCParticle *child2;

//_________________________________________________________________________
//* This constructor does not set any initial values or call any 
//* functions.

User::User() 
{
particle=NULL;
child1=NULL;
child2=NULL;
}

//_________________________________________________________________________
//* This function initializes some parameters thru function calls to
//* SetRunParameters and SetMakerParameters.  This function then creates a
//* TTree object with two branches of type TMCParticle (class type for
//* storing basic information of a particle, such as four-momentum,
//* particle code, parent, etc).

void User::Init() 
{
//* set parameters for a job

//* If the file named "run.C" exists - execute it as macro - otherwise use
//* predefined settings from SetRunParameters and SetMakerParameters
//* which choses top-antitop pair production with no smearing.
  
  if (gSystem->AccessPathName("run.C",kReadPermission) == 0 )
    gROOT->Macro("run.C");
  else {
   fprintf(stderr,"WARNING in TopMassGen::Init -> File `run.C` with ");
   fprintf(stderr,"settings does not exist.\n Using default settings.\n");
    SetRunParameters();    //change default parameters
    SetMakerParameters();  //change default parameters
    }
}

//_________________________________________________________________________
//* This function loops through each particle of the current event.  
//* If the particle is the desired particle (KF code for desired particle 
//* is passed into KFcode) originates from the desired parent (KF code
//* for desred parent particle is passed into ParentKF), its 
//* information is saved in the datamember particle of the current
//* instance of the user class. The line number of the particle (C++
//* version, ie starting at 0) is passed back to calling funcion 
//* through i.

void User::GetParticle(Int_t KFcode, Int_t ParentKF, Int_t &i)
{
//* Print out the passed in KFcode and ParentKF.  For debugging 
//* purposes.

//   cout << "LOOKING FOR:  "<<KFcode<< "  FROM:  "<<ParentKF<<endl;

//* Dummy integer for indexing particle.

   Int_t j=0;

//* This TMCParticle instance is for the parent particle.

   TMCParticle *particle2;
   particle2 = NULL;

//* Saves the event information (fruits) from the current event to a
//* TClonesArray(class for an array of identical objects, actually a 
//* sophisticated linked list) of TMCParticle's.

   ATLFMCMaker *mcmaker = gATLFast->MCMaker();
   TClonesArray *particles = mcmaker->Fruits();


//* Create an integer which sores the number of particles in the event.
 
   Int_t nparticles = particles->GetEntriesFast();

//* This forloop loops over every particle in a single event, 
//* looks for particles of type KFcode with parent of type ParentKF 
//* and saves the particle in particle.

   for (i=0; i<nparticles; i++) 
   {
//* Put particle at line number i (first particle is line number 0)
//* into particle.

     particle=(TMCParticle*)particles->At(i);

//* Makes j the line number of the parent particle.  Since this 
//* function simply returns the LUJETS value of K(3) where the
//* first particle starts at line number 1, 1 must be subtracted 
//* from j to make it the right ROOT line number.  This is a bug
//* in the ATLFast interface to Pythia.  This is used to insure
//* the parent particle is of type ParentKF.

     if (particle->GetParent()!=0)
        j=(particle->GetParent()) - 1;
     else
//* If the Parent is 0, then don't subtract 1, or else it will have
//* an illegal array index 
        j=(particle->GetParent());

//* Put particle at corrected line number j (Parent Particle) into
//* particle2.

     particle2=(TMCParticle*)particles->At(j);

//* Since j is 0 if the parent is the beam particle or if there is
//* no parent, here is an if statement for the special case of 
//* looking for a particle with no parent.  If there is a match
//* break out of the loop so the current particle is the one that
//* will be saved.

     if ((ParentKF==0) && (particle->GetKF()==KFcode) &&
        (particle->GetParent()==0))
        {
        cout << nparticles << ", " << i << ", " << 
             particle->GetKF() << " FOUND IN GETW" << endl;

          break;
        }
//* If the current particle satisfies KFcode and ParentKF conditions,
//* break out of the loop so the current particle is the one that
//* will be saved.

     if ((particle->GetKF()==KFcode)&&(abs(particle2->GetKF())==ParentKF))
        {
        cout << nparticles << ", " << i << ", " << 
             particle->GetKF() << " FOUND IN GETW" << endl;

          break;
        }
   } //end of for loop

//* If every particle in the event was looped over check to see if the
//* last particle was the desired particle.  If it wasn't, don't save
//* it.  i=-99 is a flag value for the line number of the current 
//* particle.
   if ((i==(nparticles))&&(particle->GetKF()!=KFcode))
     {
     i=-99;
     particle = NULL;
     }
}

//_________________________________________________________________________
//* This function loops through each particle of the current event.  
//* If the particle is the desired particle originates from the desired 
//* parent (the line number for desred parent particle is passed into i),
//* its information is saved in the datamember child1 or child2 of the
//* current instance of the user class, depending on whether or not it is 
//* the first or second child found. The line number of each particle (C++
//* version, ie starting at 0) is passed back to calling funcion 
//* through line1 and line2.

void User::GetChildren(Int_t i, Int_t &line1, Int_t &line2)
{
//* Temporaty TMCParticle object for looping through the particles.

   TMCParticle *temp;	

//* Nubmer of particles found to date is saved in ChildNum

   Int_t ChildNum=0;

//* Saves the event information (fruits) from the current event to a
//* TClonesArray(class for an array of identical objects, actually a 
//* sophisticated linked list) of TMCParticle's.

   ATLFMCMaker *mcmaker = gATLFast->MCMaker();
   TClonesArray *particles = mcmaker->Fruits();

//* Create an integer which sores the number of particles in the event.
 
   Int_t nparticles = particles->GetEntriesFast();

//* Dummy variable for looping through particles.

   Int_t j=0;

//* Check to make sure the parent line number is valid.
   if (i<0) 
     {
//* Flag value of line1 and line2.
     line1=-99;
     line2=-99;
     child1 = NULL;
     child2 = NULL;
     cout <<"INVALID PARENT-- NO CHILDREN SAVED"
          << endl;
     return;
     }

//* This for loop loops over every particle in a single event, 
//* looks for particles with parent at line number i
//* and saves the first two children into child1 and child2.

   for (j=0; j<nparticles; j++) 
   {
//* Put particle at line number j (first particle is line number 0)
//* into temp.

     temp=(TMCParticle*)particles->At(j);

//* If the line number of the parent particle is equal to i, than 
//* save this particle.  If it is the first child found save it
//* in child1.  If it is the second child found than save it in  
//* child2 and break out of the loop. Since this function simply 
//* returns the LUJETS value of K(3) where the first particle starts at
//* line number 1, 1 must be added to the desired parent number to make it
//* the right LUJETS line number.  This is a bug in the ATLFast interface
//* to Pythia.  

     if (temp->GetParent()==(i+1)) ChildNum++;
     if (ChildNum==1) {
        child1=temp;
        line1=j;
        cout << nparticles << ", " << j << ", " << 
           particle->GetKF() << ", " << 
           child1->GetParent() <<  endl;
        
     }
     if (ChildNum==2) {
        child2=temp;
        line2=j;
        cout << nparticles << ", " << j << ", " << 
           particle->GetKF() << ", " << 
           child2->GetParent() << endl;
        break;
     }
   }	
   
//* If less than two children were found, don't save any children.
   if ((j==(nparticles-1))&&(abs(child2->GetParent())!=i+1))
     {
     j=-99;
     child1 = NULL;
     child2 = NULL;
     cout <<"DID NOT DECAY INTO 2+ PARTICLES-- NO CHILDREN SAVED"
          << endl;
     }
}

//_________________________________________________________________________
//* This function is used for filtering events in the ATLFast Tree.
//* I accept all events.  I don't think this affects the user tree
//* at all, but I could be wrong.

void  User::Filter(Int_t &kFilter)
{
   // Accept all events

    kFilter = 1;

}

//_________________________________________________________________________
void User::SetRunParameters()
//* This is where the Phythia cards are set.  Also, the random number 
//* seed, the Hadronization Mode and ClusterMaker parameters are set. 
{
//* change ATLfast default parameters

//* This sets the mode to "WH, H-->bb, W-->lnu"
//* See: 
//* http://root.cern.ch/root/html/atlfast/src/ATLFMCMaker.cxx.html
//* for other options.

//   gATLFast->MCMaker()->SetProductionMode(7);

//* Turn off default event types.  I think this is done so that 
//* the exact Pythia cards can be set, but I cannot find any 
//* documentation on this, or event source code for this option.

   gATLFast->MCMaker()->SetProductionMode(0);  

//* The following code chooses the event type; use PYTHIA options 
//* to choose ttbar events.

//* Create a TPythia object to run Pythia and set Pythia Cards.

   TPythia *pythia = new TPythia();

//* See the Pythia Manual for more information on the Pythia cards.
//* Equivalent Pythia Commands are provided here.

   pythia->SetMSEL(0);			//MSEL=0
   pythia->SetMSUB(81,1);		//MSUB(81)=1
   pythia->SetMSUB(82,1);		//MSUB(82)=1
//   pythia->SetMSUB(2,1);		//MSUB(2)=1
  
   pythia->SetMSTP(48,2);
   pythia->SetMSTP(51,5);
   pythia->SetMSTP(61,0);
   pythia->SetMSTP(71,0);
   pythia->SetMSTP(81,0);

   for(Int_t i=172; i<192 ;i++)		//Example:
     pythia->SetMDME(i,1,1);		//MDME(172,1)=1

   for(i=41; i<48 ;i++)		//Example:
     pythia->SetMDME(i,1,0);		//MDME(172,1)=0

   pythia->SetMDME(46,1,1);		//MDME(46,1)=1

   pythia->SetMDME(188,1,1);		//MDME(188,1)=1
   pythia->SetMDME(189,1,1);		//MDME(189,1)=1

//* set hadronisation type, random number seed

//* This is the default Hadronisation Mode.  It set the mode to
//* "ISR+FSR+HP+HADRONIZE".
//* See: 
//* http://root.cern.ch/root/html/atlfast/src/ATLFMCMaker.cxx.html
//* for other options.

   gATLFast->MCMaker()->SetHadronisationMode(3);  

//* This is the default random number seed (51111).  I do not know what
//* the valid range for random numbers is, but I think it's the same as
//* the Pythia range (between 0 and 900000000, I think). 

   gATLFast->MCMaker()->SetRndNumber(51111);

//* I can't find any documentation on the Save member function, so
//* I don't know the difference between the following two lines of
//* code.  However, the ClusterMaker deposites the energy of 
//* generated particles into the calorimeter and reconstructs clusters.  I
//* assume that it is not being used here.   

   gATLFast->ClusterMaker()->Save(0);

//* Save event kinematics for use with event display 

   gATLFast->MCMaker()->Save(); 
   
}
//_________________________________________________________________________
//* All this does is set the Muon Smear Option to 2.  This means the Smear
//* Option is approximate.  Choose 1 for exact.  That's all I know.

void User::SetMakerParameters()
{
//* change ATLfast default parameters
   
   gATLFast->SetSmearMuonOpt(1);
   gATLFast->SetSmearMuonFun(1);
   gATLFast->SetSmearing(0);
}

//_________________________________________________________________________
TMCParticle* User::CurrentParticle()
{
return particle;
}
//_________________________________________________________________________
TMCParticle* User::CurrentFirstChild()
{
return child1;
}
//_________________________________________________________________________
TMCParticle* User::CurrentSecondChild()
{
return child2;
}


