AUTHOR:  Sarah D. Groulx	LAST MODIFIED:  June 30th, 1998

//* This is where the Phythia cards are set.  Also, the random number
//* seed, the Hadronization Mode and ClusterMaker parameters are set.
{
//* change ATLfast default parameters

//* load ATLFAST libraries
  gROOT->Macro("$ATLFAST/loadlibs");


//* Turn off default event types.  I think this is done so that
//* the exact Pythia cards can be set, but I cannot find any
//* documentation on this, or event source code for this option.

  gATLFast->MCMaker()->SetProductionMode(0);


//* The following code chooses the event type; use PYTHIA options
//* to choose Drell-Yan W events.

//* Set Pythia parameters:

/*****************************************************************************
  // This is a bit tricky!
  // We must have an access to Pythia generator before Atlfast->Init()
  // is called. But TPythia object is normally created in Atlfast->Init() !!!
  // It doesn't matter, because we always have only one generator - and every
  // TPythia object is just another interfacing object to this generator.
  // so: If we create a TPythia object here, we create an interface to 
  // generator , and we can use this instance of TPythia interface to set our
  // switches. It doesn't matter that ATLFast will create another TPythia
  // object - it will create just another interface to THE SAME Pythia 
  // generator, not changing anything without our commitment. And of course
  // it doesn't matter that it will use its own interface - not this one...
  // Once again:
  // using gATLFast->MCMaker->Generator() would return NULL pointer , because
  // TPythia object is not created in ATLFast::ATLFast(). We create just
  // another interfacing object to communicate with Pythia - we can make it
  // local - it won't be useful after exiting this routine.
******************************************************************************/

//* Create a TPythia object to run Pythia and set Pythia Cards.

  TPythia *pythia = new TPythia();

//* See the Pythia Manual for more information on the Pythia cards.
//* Equivalent Pythia Commands are provided here.

  pythia->SetMSEL(0); 		//MSEL=0	Processes must be switched
				//	        on in MSUB.

//* This choses top-antitop production events.
//  pythia->SetMSUB(81,1);	//MSUB(81)=1
//  pythia->SetMSUB(82,1);	//MSUB(82)=1

//* This choses Drell-Yan W production.
  pythia->SetMSUB(2,1);		//MSUB(2)=1
  
  pythia->SetMSTP(48,2);	//MSTP(48)=2	top decays before
				//		fragmentation
  pythia->SetMSTP(51,5);	//MSTP(51)=5	parton-distribution
				//		density=5
  pythia->SetMSTP(61,0);	//MSTP(61)=0	turn initial state
				//		radiation off
  pythia->SetMSTP(71,0);	//MSTP(71)=0	turn final state radiation
				//		on
  pythia->SetMSTP(81,0);	//MSTP(81)=0	turn multiple interactions
				//		off

//* Switches for possible decays.
  for(Int_t i=172; i<192 ;i++)  	//Example:  
     pythia->SetMDME(i,1,1);            //MDME(172,1)=1


  for(i=41; i<48 ;i++)                  //Example:
     pythia->SetMDME(i,1,0);            //MDME(41,1)=0

  pythia->SetMDME(46,1,1);              //MDME(46,11)=1

  pythia->SetMDME(188,1,1);            //MDME(188,1)=1
  pythia->SetMDME(189,1,1);            //MDME(189,1)=1
     

//* This is the default Hadronisation Mode.  It set the mode to
//* "ISR+FSR+HP+HADRONIZE".
//* See:
//* http://root.cern.ch/root/html/atlfast/src/ATLFMCMaker.cxx.html
//* for other options. 
 
  gATLFast->MCMaker()->SetHadronisationMode(3);

//* This changes the random number seed (default=51111).  I don't know
//* what the valid range for random numbers is, but I think it's the same
//* as the Pythia range (between 0 and 900000000, I think).

  gATLFast->MCMaker()->SetRndNumber(51112);


//* change ATLfast makers' parameters


//* Turn atlfast smearing off
  gATLFast->SetSmearing(0);
//* Make muon smearing exact (ie no smearing)
  gATLFast->SetSmearMuonFun(1);
//* If there is muon smearing, smear tracks.
  gATLFast->SetSmearMuonOpt(2);

//* Save event kinematics for use with event display

  gATLFast->ClusterMaker()->Save(0);
  gATLFast->TrackMaker()->Save(0);
  gATLFast->MCMaker()->Save(1);
}
